<?php

namespace Drupal\ohdear_integration\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Http\RequestStack;
use Drupal\Core\Session\AccountInterface;
use Drupal\ohdear_integration\OhdearHealthcheckGenerator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Returns responses for Oh Dear Integration routes.
 */
class OhdearIntegrationController extends ControllerBase {

  /**
   * Oh dear integration configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $ohdearSettings;

  /**
   * Healthcheck generator service.
   *
   * @var \Drupal\ohdear_integration\OhdearHealthcheckGenerator
   */
  protected $ohdearGenerator;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs oh dear integration controller.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $ohdear_settings
   *   The immutable config.
   * @param \Drupal\ohdear_integration\OhdearHealthcheckGenerator $ohdear_generator
   *   The ohdear helper objects factory service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The simple sitemap extensions logger service.
   * @param \Drupal\Core\Http\RequestStack $request_stack
   *   The request stack service.
   */
  public function __construct(ImmutableConfig $ohdear_settings, OhdearHealthcheckGenerator $ohdear_generator, LoggerChannelInterface $logger, RequestStack $request_stack) {
    $this->ohdearSettings = $ohdear_settings;
    $this->ohdearGenerator = $ohdear_generator;
    $this->logger = $logger;
    $this->currentRequest = $request_stack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): OhdearIntegrationController {
    return new static(
      $container->get('config.factory')->get('ohdear_integration.settings'),
      $container->get('ohdear_healthcheck.generator'),
      $container->get('logger.channel.ohdear_integration'),
      $container->get('request_stack')
    );
  }

  /**
   * Builds the response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response.
   */
  public function buildJson() {
    return new JsonResponse($this->ohdearGenerator->getData(), 200, [], TRUE);
  }

  /**
   * Access result callback.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Determines the access to controller.
   */
  public function access(AccountInterface $account) {
    $has_permission = $account->hasPermission('monitoring reports');
    $ohdear_header = $this->currentRequest->headers->get('oh-dear-health-check-secret') ?? NULL;
    if ($ohdear_header === $this->ohdearSettings->get('ohdear_healthcheck_secret') || $has_permission) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

}
